@@ -335,15 +335,15 @@ spec_arimaCoefF <- function(enabled=NA, armaP=NA, armaF=NA , coefP=NA, coefF=NA)
 }
 
 spec_seasma <- function(seasma=NA){
-
   len <- length(seasma)
   seasma.type <- c("Msr","Stable","X11Default","S3X1","S3X3","S3X5","S3X9","S3X15")
-  if (sum(is.na(seasma))!=0){
+  if (sum(is.na(seasma)) != 0) {
     return(NA)
-  } else if (!is.vector(seasma)|is.list(seasma)|length(setdiff(seasma,seasma.type))>0){
-      warning("wrong format of the x11.seasonalma.\nPossisble filters per period: \"Msr\",\"Stable\", \"X11Default\", \"S3X1\", \"S3X3\", \"S3X5\", \"S3X9\" and \"S3X15\".\nPre-specified seasonal filters will be ignored.", call. = FALSE)
+  } else if (!is.vector(seasma) | is.list(seasma) |
+             length(setdiff(seasma,seasma.type)) > 0) {
+      warning("wrong format of the x11.seasonalma.\nPossibles filters per period: \"Msr\",\"Stable\", \"X11Default\", \"S3X1\", \"S3X3\", \"S3X5\", \"S3X9\" and \"S3X15\".\nPre-specified seasonal filters will be ignored.", call. = FALSE)
       return(NA)
-  } else if (!(len %in% c(1,2,4,12))){
+  } else if (!(len %in% c(1, 2, 4, 6, 12))) {
     warning("wrong format of the x11.seasonalma.\nPre-specified seasonal filters will be ignored.", call. = FALSE)
     return(NA)
   } else {
@@ -351,7 +351,46 @@ spec_seasma <- function(seasma=NA){
     return(z)
   }
 }
+spec_calendar_sigma <- function(calendarSigma = NA, sigmaVector = NA){
+  len <- length(calendarSigma)
+  calendarSigma.type <- c("None","Signif","All","Select")
+  sigmaVector.type <- c("Group1", "Group2")
 
+  if (identical_na(calendarSigma)) {
+    if (!identical_na(sigmaVector)) {
+      warning("x11.sigmaVector will be ignored: x11.calendarSigma has to be set to \"Select\"", call. = FALSE)
+    }
+    calendarSigma <- sigmaVector <- NA
+  } else if (is.list(calendarSigma) ||
+             length(calendarSigma) > 1 ||
+             !calendarSigma %in% calendarSigma.type) {
+    warning("Wrong format of the x11.calendarSigma.",
+            "\nPossibles values are: ",
+            "\"None\",\"Signif\", \"All\", \"Select\".",
+            "\nParameters will be ignored.", call. = FALSE)
+    calendarSigma <- sigmaVector <- NA
+  } else if (identical(calendarSigma, "Select")) {
+    if (identical_na(sigmaVector)) {
+      warning("x11.sigmaVector must be specified when x11.calendarSigma = \"Select\"." ,
+              "\nx11.calendarSigma will be set to ",
+              '"None".', call. = FALSE)
+      sigmaVector <- NA
+      calendarSigma <- "None"
+    }else if (length(setdiff(sigmaVector, sigmaVector.type)) > 0 ||
+        !(length(sigmaVector) %in% c(2, 4, 6, 12))) {
+      warning("Wrong format of the x11.sigmaVector." ,
+              "\nIt will be ignored and x11.calendarSigma is set to ",
+              '"None".', call. = FALSE)
+      sigmaVector <- NA
+      calendarSigma <- "None"
+    }else{
+      sigmaVector <- toString(sigmaVector)
+    }
+  } else {
+    sigmaVector <- NA
+  }
+  list(calendarSigma = calendarSigma, sigmaVector = sigmaVector)
+}
 spec_trendma <- function(trendma=NA){
 
   if (sum(!is.na(trendma))==0){
@@ -412,42 +451,47 @@ spec_transformX13<-function(trans){
 
 spec_tdX13<-function(td, tf, tadj){
 
-  td[3,1] <- if(!is.na(td[2,1])) {td[2,1]} else {td[1,1]}
-  td[3,4] <- if(!is.na(td[2,4])) {td[2,4]} else {td[1,4]}
+  td[3, "option"] <- if(!is.na(td[2, "option"])) {td[2, "option"]} else {td[1, "option"]}
+  td[3, "stocktd"] <- if(!is.na(td[2, "stocktd"])) {td[2, "stocktd"]} else {td[1, "stocktd"]}
 
-  if (td[3,1]=="None" & td[3,4]==0) {
-    td[3,2]<- td[1,2]
-    td[3,c(3,5)]<- "None"
-  } else if (td[3,1]=="None") {
-    td[3,2]<- td[1,2]
-    td[3,3]<- "None"
+  if (td[3, "option"]=="None" & td[3, "stocktd"]==0) {
+    td[3, "autoadjust"]<- td[1, "autoadjust"]
+    td[3,c("leapyear", "test")]<- "None"
+  } else if (td[3, "option"]=="None") {
+    td[3, "autoadjust"]<- td[1, "autoadjust"]
+    td[3, "leapyear"]<- "None"
   } else {
-    td[3,4] <-0
+    td[3, "stocktd"] <-0
   }
-  if (is.na(td[3,2])){
-    td[3,2] <- if(!is.na(td[2,2])) {td[2,2]} else {td[1,2]}
+  if (is.na(td[3, "autoadjust"])){
+    td[3, "autoadjust"] <- if(!is.na(td[2, "autoadjust"])) {td[2, "autoadjust"]} else {td[1, "autoadjust"]}
 
-    if (td[3,2] & as.character(tf)=="Auto") {
-      td[3,3]<- td[1,3]
+    if (td[3, "autoadjust"] & as.character(tf)=="Auto") {
+      td[3, "leapyear"]<- td[1, "leapyear"]
     }else{
-      td[3,2]<-FALSE
+      td[3, "autoadjust"]<-FALSE
     }
   }
-  if (is.na(td[3,3])){
+  if (is.na(td[3, "leapyear"])){
     if (tadj!="None") {
-      td[3,3] <- "None"
+      td[3, "leapyear"] <- "None"
     }else{
-      td[3,3] <- if(!is.na(td[2,3])) {td[2,3]} else {td[1,3]}
+      td[3, "leapyear"] <- if(!is.na(td[2, "leapyear"])) {td[2, "leapyear"]} else {td[1, "leapyear"]}
     }
   }
-  if (is.na(td[3,5]))
-    td[3,5] <- if(!is.na(td[2,5])) {td[2,5]} else {td[1,5]}
+  if (is.na(td[3, "test"]))
+    td[3, "test"] <- if(!is.na(td[2, "test"])) {td[2, "test"]} else {td[1, "test"]}
 
   #UserDefined TD regressors
-  if(td[3,1] == "UserDefined"){
-    td[3,3] <- "None"
-    td[3,2] <- FALSE
-    td[3,4] <- 0
+  if(td[3, "option"] == "UserDefined"){
+    if(any(!is.na(td[1, c("autoadjust", "leapyear", "stocktd")]))){
+      warning("With tradingdays.option = \"UserDefined\", the parameters tradingdays.autoadjust, tradingdays.leapyear and tradingdays.stocktd are ignored.\n",
+              call. = FALSE)
+    }
+
+    td[3, "leapyear"] <- "None"
+    td[3, "autoadjust"] <- FALSE
+    td[3, "stocktd"] <- 0
   }
   rownames(td) <- c("Predefined","User_modif","Final")
   return(td)
@@ -596,33 +640,38 @@ spec_transformTS<-function(trans){
 
 spec_tdTS<-function(td){
 
-  td[3,1] <- if(!is.na(td[2,1])) {td[2,1]} else {td[1,1]}
+  td[3, "automatic"] <- if(!is.na(td[2, "automatic"])) {td[2, "automatic"]} else {td[1, "automatic"]}
 
-  if (td[3,1]!= "Unused"){
-    td[3,2] <- if(!is.na(td[2,2])) {td[2,2]} else {td[1,2]}
-    td[3,3:6] <- td[1,3:6]
+  if (td[3, "automatic"]== "Unused"){
+    td[3, "pftd"] <- if(!is.na(td[2, "pftd"])) {td[2, "pftd"]} else {td[1, "pftd"]}
+    td[3, c("option", "leapyear", "stocktd", "test")] <- td[1, c("option", "leapyear", "stocktd", "test")]
   }else{
-    td[3,2] <-td[1,2]
-    td[3,3] <- if(!is.na(td[2,3])) {td[2,3]} else {td[1,3]}
-    td[3,5] <- if(!is.na(td[2,5])) {td[2,5]} else {td[1,5]}
+    td[3, "pftd"] <-td[1, "pftd"]
+    td[3, "option"] <- if(!is.na(td[2, "option"])) {td[2, "option"]} else {td[1, "option"]}
+    td[3, "stocktd"] <- if(!is.na(td[2, "stocktd"])) {td[2, "stocktd"]} else {td[1, "stocktd"]}
 
-    if (td[3,3]=="None" & td[3,5]==0){
-      td[3,4] <- FALSE
-      td[3,6] <- "None"
-    }else if (td[3,3]=="None"){
-     td[3,4] <- FALSE
-      td[3,6] <- if(!is.na(td[2,6])) {td[2,6]} else {td[1,6]}
+    if (td[3, "option"]=="None" & td[3, "stocktd"]==0){
+      td[3, "leapyear"] <- FALSE
+      td[3, "test"] <- "None"
+    }else if (td[3, "option"]=="None"){
+     td[3, "leapyear"] <- FALSE
+      td[3, "test"] <- if(!is.na(td[2, "test"])) {td[2, "test"]} else {td[1, "test"]}
     }else{
-      td[3,5]<-0
-      td[3,4] <- if(!is.na(td[2,4])) {td[2,4]} else {td[1,4]}
-      td[3,6] <- if(!is.na(td[2,6])) {td[2,6]} else {td[1,6]}
+      td[3, "stocktd"]<-0
+      td[3, "leapyear"] <- if(!is.na(td[2, "leapyear"])) {td[2, "leapyear"]} else {td[1, "leapyear"]}
+      td[3, "test"] <- if(!is.na(td[2, "test"])) {td[2, "test"]} else {td[1, "test"]}
     }
   }
 
   #UserDefined TD regressors
-  if(td[3,3] == "UserDefined"){
-    td[3,4] <- FALSE
-    td[3,5] <- 0
+  if(td[3, "option"] == "UserDefined"){
+    if(any(!is.na(td[1, c("automatic","leapyear", "stocktd")]))){
+      warning("With tradingdays.option = \"UserDefined\", the parameters tradingdays.leapyear and tradingdays.stocktd are ignored.\n",
+              call. = FALSE)
+    }
+    td[3, "automatic"] <- td[1, "automatic"]
+    td[3, "leapyear"] <- FALSE
+    td[3, "stocktd"] <- 0
   }
 
   rownames(td) <- c("Predefined","User_modif","Final")
@@ -726,7 +775,7 @@ spec_arimaTS <-function(arimaspc, arimaco){
 
 # Common for X-13 and TRAMO-SEATS
 
-spec_userdef <-function(usrspc, out, var, tf) {
+spec_userdef <- function(usrspc, out, var, tf) {
   outF <- out$Final
   outP <- out$Predefined
   varF <- var$Final
@@ -791,7 +840,7 @@ spec_userdef <-function(usrspc, out, var, tf) {
   return(x)
 }
 
-spec_forecast <-function(fcst){
+spec_forecast <- function(fcst){
   fcst[3,1] <- if (!is.na(fcst[2,1])) {fcst[2,1]} else {fcst[1,1]}
   rownames(fcst) <- c("Predefined","User_modif","Final")
   return(fcst)
@@ -799,22 +848,30 @@ spec_forecast <-function(fcst){
 
 # X11/ SEATS
 
-spec_x11 <- function(x11spc){
+spec_x11 <- function(x11){
 
-  x11 <- x11spc
-
-  for (i in c(1:5,8:10)){
-    x11[3,i] <- if(!is.na(x11[2,i])) {x11[2,i]} else {x11[1,i]}
+  for (i in c("x11.mode", "x11.seasonalComp", "x11.lsigma", "x11.usigma",
+              "x11.trendAuto", "x11.fcasts", "x11.bcasts", "x11.excludeFcasts")
+  ) {
+    x11[3,i] <- if (!is.na(x11[2,i])) {x11[2,i]} else {x11[1,i]}
   }
-  if(x11[3,5] | is.na(x11[2,6])){
-    x11[3,6]<- x11[1,6]
+  if (x11[3,"x11.trendAuto"] | is.na(x11[2, "x11.trendma"])) {
+    x11[3, "x11.trendma"] <- x11[1, "x11.trendma"]
   }else {
-    x11[3,6] <- x11[2,6]
+    x11[3,"x11.trendma"] <- x11[2, "x11.trendma"]
   }
-  if(x11[3,2]==FALSE | is.na(x11[2,7])){
-    x11[3,7] <- x11[1,7]
+  if(x11[3,"x11.seasonalComp"] | is.na(x11[2,"x11.seasonalma"])){
+    x11[3,"x11.seasonalma"] <- x11[1,"x11.seasonalma"]
   }else {
-    x11[3,7] <- x11[2,7]
+    x11[3,"x11.seasonalma"] <- x11[2,"x11.seasonalma"]
+  }
+  if (is.na(x11[2, "x11.calendarSigma"])) {
+    x11[3, c("x11.calendarSigma","x11.sigmaVector")] <- x11[1, c("x11.calendarSigma","x11.sigmaVector")]
+  } else if (!identical(x11[2, "x11.calendarSigma"], "Select")){
+    x11[3, "x11.calendarSigma"] <- x11[2, "x11.calendarSigma"]
+    x11[3, "x11.sigmaVector"] <- NA
+  } else{
+    x11[3, c("x11.calendarSigma","x11.sigmaVector")] <- x11[2, c("x11.calendarSigma","x11.sigmaVector")]
   }
 
   rownames(x11) <- c("Predefined","User_modif","Final")
@@ -824,8 +881,8 @@ spec_x11 <- function(x11spc){
 spec_seats <- function(seatspc){
   seats <- seatspc
 
-  for (i in c(1:7)){
-    seats[3,i] <- if(!is.na(seats[2,i])) {seats[2,i]} else {seats[1,i]}
+  for (i in seq_len(ncol(seats))) {
+    seats[3,i] <- if (!is.na(seats[2,i])) {seats[2,i]} else {seats[1,i]}
   }
   rownames(seats) <- c("Predefined","User_modif","Final")
   return(seats)
